#region Using declarations
using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// DoubleMA
    /// </summary>
    [Description("DoubleMA")]
    [Gui.Design.DisplayName("Double MA")]
    public class DoubleMA : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int mA2Period = 1; // Default setting for HMAPeriod
            private int mA1Period = 1; // Default setting for MAPeriod
			private NinjaTrader.Indicator.MAV.MAType mA1Type = NinjaTrader.Indicator.MAV.MAType.EMA;
			private NinjaTrader.Indicator.MAV.MAType mA2Type = NinjaTrader.Indicator.MAV.MAType.EMA;
			private Color risingColor;
			private Color fallingColor;
			private bool showArrows = true;
			private int arrowDisplacement = 7;
		// User defined variables (add any user defined variables below)
			private int direction;
			private int lastDirection;
			private int lastCalcBar = -2;
			private bool drawObjects = false;
			private ArrayList data = new ArrayList();

			private int decimals;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot( Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "RisingPlot") );
            Add(new Plot( Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "FallingPlot") );
            Add(new Plot(Color.FromKnownColor(KnownColor.Yellow), PlotStyle.Line, "NeutralPlot"));
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
			//triEma = EMA( EMA( EMA( this, Period ), Period ), Period );
			lastDirection = -2;
			double double1 = TickSize;
			int max = 30;
			int i = 0;
			while (double1 % 1 !=0 )
			{
				double1 *= 10;
				decimals++;
				i++;
				if( i > max )
					break;
			}
			if( decimals > 0 )
				decimals++;
			//Print( TickSize + " dec " + decimals );
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			try
			{
				if( lastCalcBar != CurrentBar )
				{
					lastCalcBar = CurrentBar;
					setData( data, 0, 0d );
					colorBar( 1 );
					lastDirection = direction;
				}
				if (CurrentBar < 1)
				{
					risingColor = this.Plots[0].Pen.Color;
					fallingColor = this.Plots[1].Pen.Color;
					setData( data, 0, 0.0d);
					return;
				}
				colorBar( 0 );
			}
			catch( Exception ex )
			{
				Print( ex.ToString() );
			}

			
        }


		private void colorBar( int bar )
		{
			try
			{
				direction = 0;
				double val1Pure = MAV( MAV(Input, MA1Period, (int)MA1Type ), MA2Period, (int)MA2Type)[ bar ];
				double val2Pure = MAV( MAV(Input, MA1Period, (int)MA1Type ), MA2Period, (int)MA2Type)[ bar + 1 ];
				double val1 = Math.Round( val1Pure, decimals );
				double val2 = Math.Round( val2Pure, decimals );
				bool rising = false;
				bool falling = false;
				if( val1 > val2 )
					rising = true;
				if( val1 < val2 )
					falling = true;
				
				if( Rising( MAV( MAV(Input, MA1Period, (int)MA1Type ), MA2Period, (int)MA2Type) ) )
					direction = 1;
				else if( Falling( MAV( MAV(Input, MA1Period, (int)MA1Type ), MA2Period, (int)MA2Type) ) )
					direction -= 1;
				else
					direction = 0;

				
				if(rising)
				{
					RisingPlot.Set(bar + 1, val2);
					RisingPlot.Set(bar, val1);
				}
				else if(falling)
				{
					FallingPlot.Set(bar + 1, val2);
					FallingPlot.Set(bar, val1);
				}
				else
				{
					NeutralPlot.Set(bar + 1, val2);
					NeutralPlot.Set(bar, val1);
				}

				if( ShowArrows && CurrentBar > 2 )
				{
					if( lastDirection < direction )
					{
						double val = Low[bar+1] - ( TickSize * ArrowDisplacement );
						//DrawArrowUp(CurrentBar.ToString(),1, val , risingColor );
						//data[ data.Count - 1] = val  * -1;
						setData( data, bar, val  * -1 );
					}
					else if( lastDirection > direction )
					{
						double val = High[bar+1] + ( TickSize * ArrowDisplacement );
						//DrawArrowDown(CurrentBar.ToString(),1, val, fallingColor );
						setData( data, bar, val );
					}	
				}
			}
			catch( Exception ex )
			{
				Print( ex.ToString() );
			}
		}

		private void setData( ArrayList list, int index, double val )
		{
			if( list == null || index < 0 )
				return;
			if( list.Count - 1 >= CurrentBar )
				list[ list.Count - index - 1 ] = val;
			else if( index == 0 )
				list.Add( val );
		}

		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			// Default plotting in base class. 
			base.Plot(graphics, bounds, min, max);
			
			if (base.Bars == null || !ShowArrows) return;
			
			
        	int index = -1;
			Exception caughtException;
			
			
				
            int barPaintWidth = base.ChartControl.ChartStyle.GetBarPaintWidth(base.ChartControl.BarWidth);
			int triangleWidth = 10;
			int arrowHeight = 15;
			int barWidth = 4;
			int bars = base.ChartControl.BarsPainted;
            while (bars >= 0)
            {
				index = ((base.ChartControl.LastBarPainted - base.ChartControl.BarsPainted) + 1) + bars;
                if (base.ChartControl.ShowBarsRequired || ((index - base.Displacement) >= base.BarsRequired))
                {
					try
					{
						if( data.Count > index && (double)data[ index ] != 0 )
						{
							double val = (double)data[ index ];
							int dir = 1;
							if( val < 0 )
							{
								val = val * -1;
								dir = -1;
							}
							
							Brush brush;
							int x1 = (((base.ChartControl.CanvasRight - base.ChartControl.BarMarginLeft - barPaintWidth ) ) - ((base.ChartControl.BarsPainted - 1) * base.ChartControl.BarSpace)) + (bars * base.ChartControl.BarSpace);
							int y1 = (bounds.Y + bounds.Height) - ((int) (((val - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height));
							int triangleTip;
							//int y2 = (bounds.Y + bounds.Height) - ((int) (((Values[2].Get(index) - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height));
							if( barPaintWidth > triangleWidth )
							{
								x1 += ( barPaintWidth - triangleWidth ) / 2;
							}
							else if( barPaintWidth < triangleWidth )
							{
								x1 -= ( triangleWidth - barPaintWidth ) / 2;
							}
							y1 = y1 + (int)( 0.5 * ( ( dir * -1 ) * triangleWidth ) );

							if( dir > 0 )
							{
								brush = new SolidBrush( fallingColor );
								triangleTip = y1 + triangleWidth / 2;
							}
							else
							{
								triangleTip = y1 - triangleWidth / 2;
								brush = new SolidBrush( risingColor );
							}
							Point[] points = new Point[]{
								      new Point( x1, y1)
									, new Point( x1 + ( ( triangleWidth - barWidth ) / 2 ), y1 )
									, new Point( x1 + ( ( triangleWidth - barWidth ) / 2 ), y1 + ( dir * -1 * arrowHeight ) )
									, new Point( x1 + barWidth + ( ( triangleWidth - barWidth ) / 2 ), y1 + ( dir * -1 * arrowHeight ) )
									, new Point( x1 + barWidth + ( ( triangleWidth - barWidth ) / 2 ), y1 )
									, new Point( x1 + triangleWidth, y1 )
									, new Point( x1 + ( triangleWidth / 2 ), triangleTip ) };
							graphics.FillPolygon( brush, points );
							Pen pen = new Pen( Color.FromKnownColor( KnownColor.Black ) );
							graphics.DrawPolygon( pen, points );
						}
					}
					catch (Exception exception) { caughtException = exception; Print( exception.ToString() ); }
					
               	}
				bars--;					
			}
		}

		#region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries RisingPlot
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries FallingPlot
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries NeutralPlot
        {
            get { return Values[2]; }
        }

        [Description("")]
        [Category("Parameters")]
        public int MA2Period
        {
            get { return mA2Period; }
            set { mA2Period = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int MA1Period
        {
            get { return mA1Period; }
            set { mA1Period = Math.Max(1, value); }

        }

		[Description("Moving Average Type")]
        [Category("Parameters")]
		public NinjaTrader.Indicator.MAV.MAType MA1Type
		{
			get { return mA1Type; }
            set { mA1Type = value; }
		}

		[Description("Moving Average Type")]
        [Category("Parameters")]
		public NinjaTrader.Indicator.MAV.MAType MA2Type
		{
			get { return mA2Type; }
            set { mA2Type = value; }
		}
		
        [Description("Show Arrows")]
        [Category("Plots")]
        public bool ShowArrows
        {
            get { return showArrows; }
            set { showArrows = value; }
        }

        [Description("Draw Arrows above/below bars")]
        [Category("Plots")]
        public int ArrowDisplacement
        {
            get { return arrowDisplacement; }
            set { arrowDisplacement = Math.Max( 0, value ); }
        }
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DoubleMA[] cacheDoubleMA = null;

        private static DoubleMA checkDoubleMA = new DoubleMA();

        /// <summary>
        /// DoubleMA
        /// </summary>
        /// <returns></returns>
        public DoubleMA DoubleMA(int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            return DoubleMA(Input, mA1Period, mA1Type, mA2Period, mA2Type);
        }

        /// <summary>
        /// DoubleMA
        /// </summary>
        /// <returns></returns>
        public DoubleMA DoubleMA(Data.IDataSeries input, int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            checkDoubleMA.MA1Period = mA1Period;
            mA1Period = checkDoubleMA.MA1Period;
            checkDoubleMA.MA1Type = mA1Type;
            mA1Type = checkDoubleMA.MA1Type;
            checkDoubleMA.MA2Period = mA2Period;
            mA2Period = checkDoubleMA.MA2Period;
            checkDoubleMA.MA2Type = mA2Type;
            mA2Type = checkDoubleMA.MA2Type;

            if (cacheDoubleMA != null)
                for (int idx = 0; idx < cacheDoubleMA.Length; idx++)
                    if (cacheDoubleMA[idx].MA1Period == mA1Period && cacheDoubleMA[idx].MA1Type == mA1Type && cacheDoubleMA[idx].MA2Period == mA2Period && cacheDoubleMA[idx].MA2Type == mA2Type && cacheDoubleMA[idx].EqualsInput(input))
                        return cacheDoubleMA[idx];

            DoubleMA indicator = new DoubleMA();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.MA1Period = mA1Period;
            indicator.MA1Type = mA1Type;
            indicator.MA2Period = mA2Period;
            indicator.MA2Type = mA2Type;

            DoubleMA[] tmp = new DoubleMA[cacheDoubleMA == null ? 1 : cacheDoubleMA.Length + 1];
            if (cacheDoubleMA != null)
                cacheDoubleMA.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheDoubleMA = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// DoubleMA
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DoubleMA DoubleMA(int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            return _indicator.DoubleMA(Input, mA1Period, mA1Type, mA2Period, mA2Type);
        }

        /// <summary>
        /// DoubleMA
        /// </summary>
        /// <returns></returns>
        public Indicator.DoubleMA DoubleMA(Data.IDataSeries input, int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            return _indicator.DoubleMA(input, mA1Period, mA1Type, mA2Period, mA2Type);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// DoubleMA
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DoubleMA DoubleMA(int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            return _indicator.DoubleMA(Input, mA1Period, mA1Type, mA2Period, mA2Type);
        }

        /// <summary>
        /// DoubleMA
        /// </summary>
        /// <returns></returns>
        public Indicator.DoubleMA DoubleMA(Data.IDataSeries input, int mA1Period, NinjaTrader.Indicator.MAV.MAType mA1Type, int mA2Period, NinjaTrader.Indicator.MAV.MAType mA2Type)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DoubleMA(input, mA1Period, mA1Type, mA2Period, mA2Type);
        }

    }
}
#endregion
