#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Variable moving average
    /// </summary>
    [Description("Variable moving average")]
    [Gui.Design.DisplayName("MA Variable")]
    public class MAV : Indicator
    {
         #region Variables
        // Wizard generated variables
            private int mAPeriod = 1; // Default setting for MAPeriod
			private MAType mAType = MAType.HMA;
			private int typeInt;
			private bool bInit = false;
			public enum MAType			// A list of the valid types of moving averages that may be selected.
			{
				EMA = 1,					// Exponential Moving Average.
				HMA = 2,					// Hull Moving Average.
				WMA = 3,					// Weighted Moving Average.
				SMA = 4,					// Simple Moving Average.
				MAX = 5,					// Weighted Moving Average.
				MIN = 6,					// Weighted Moving Average.
				None = 7					// Weighted Moving Average.
				/*
				KAMA = 8,					// Kaufman's Adaptive Moving Average.
				LSMA = 9,					// Linear Regression = Least Squares Moving Average.
				MAMA = 10,					// MESA Adaptive Moving Average.
				TEMA = 11,					// Triple Exponential Moving Average.
				TMA = 12,					// Volume Weighted Moving Average.
				VWMA = 13 ,					// Volume Weighted Moving Average.
				VOLMA = 14					// Volume Moving Average.
				*/
			}
			Indicator indicator;
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			try
			{
				Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "MA"));
				CalculateOnBarClose	= true;
				Overlay				= true;
				PriceTypeSupported	= false;
				/*switch (mAType)
				{
					case MAType.EMA:	indicator = EMA( this, MAPeriod);break;		// Exponential Moving Average.
					case MAType.HMA:	indicator = HMA( this, MAPeriod);break;		// Hull Moving Average.
					case MAType.SMA:	indicator = SMA( this, MAPeriod);break;		// Simple Moving Average.
					case MAType.WMA:	indicator = WMA( this, MAPeriod);break;		// Weighted Moving Average.
					case MAType.MIN:	indicator = MIN( this, MAPeriod);break;		// Weighted Moving Average.
					case MAType.MAX:	indicator = MAX( this, MAPeriod);break;		// Weighted Moving Average.
					case MAType.KAMA:	indicator = KAMA( this, 1, MAPeriod, MAPeriod);break;	// Kaufman's Adaptive Moving Average.
					case MAType.LSMA:	indicator = LinReg( this, MAPeriod);break;	// Linear Regression = Least Squares Moving Average.
					case MAType.MAMA:	indicator = MAMA( this, 1, MAPeriod);break;	// MESA Adaptive Moving Average.
					case MAType.TEMA:	indicator = TEMA( this, MAPeriod);break;	// Triple Exponential Moving Average.
					case MAType.TMA:	indicator = TMA( this, MAPeriod);break;		// Triangular Moving Average.
					case MAType.VWMA:	indicator = VWMA( this, MAPeriod);break;	// Volume Weighted Moving Average.
					case MAType.VOLMA:	indicator = VOLMA( this, MAPeriod);break;	// Volume Moving Average.
				}
				*/
			}
			catch( Exception ex )
			{
				Print( ex.ToString() );
			}
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			try
			{
				
				if( mAType == MAType.SMA )
					MA.Set( SMA(Input,MAPeriod)[0] );
				else if( mAType == MAType.EMA )
					MA.Set( EMA(Input,MAPeriod)[0] );
				else if( mAType == MAType.WMA )
					MA.Set( WMA(Input,MAPeriod)[0] );
				else if( mAType == MAType.HMA )
					MA.Set( HMA(Input,MAPeriod)[0] );
				else if( mAType == MAType.MAX )
					MA.Set( MAX(Input,MAPeriod)[0] );
				else if( mAType == MAType.MIN )
					MA.Set( MIN(Input,MAPeriod)[0] );
				else
					MA.Set( Input[ 0 ] ); 
				
				/*if( indicator != null )
					MA.Set( indicator[ 0 ] );
				else
					MA.Set( Input[ 0 ] );
				*/
			}
			catch( Exception ex )
			{
				Print( ex.ToString() );
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MA
        {
            get { return Values[0]; }
        }

        [Description("")]
        [Category("Parameters")]
        public int MAPeriod
        {
            get { return mAPeriod; }
            set { mAPeriod = Math.Max(1, value); }

        }

        [Category("Parameters")]
		public int TypeInt
		{
			get { return typeInt; }
            set { 
					typeInt = Math.Max( 1, Math.Min( 7, value ) );
					if( typeInt == (int)MAType.SMA )
						mAType = MAType.SMA;
					else if( typeInt == (int)MAType.EMA )
						mAType = MAType.EMA;
					else if( typeInt == (int)MAType.WMA )
						mAType = MAType.WMA;
					else if( typeInt == (int)MAType.HMA )
						mAType = MAType.WMA;
					else if( typeInt == (int)MAType.MAX )
						mAType = MAType.MAX;
					else if( typeInt == (int)MAType.MIN )
						mAType = MAType.MIN;
					/*
					else if( typeInt == (int)MAType.KAMA )
						mAType = MAType.KAMA;
					else if( typeInt == (int)MAType.LSMA )
						mAType = MAType.LSMA;
					else if( typeInt == (int)MAType.MAMA )
						mAType = MAType.MAMA;
					else if( typeInt == (int)MAType.TEMA )
						mAType = MAType.TEMA;
					else if( typeInt == (int)MAType.TMA )
						mAType = MAType.TMA;
					else if( typeInt == (int)MAType.VOLMA )
						mAType = MAType.VOLMA;
					else if( typeInt == (int)MAType.VWMA )
						mAType = MAType.VWMA;
					*/
					else
						mAType = MAType.None;
				}
		}
		#endregion
    }
	
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private MAV[] cacheMAV = null;

        private static MAV checkMAV = new MAV();

        /// <summary>
        /// Variable moving average
        /// </summary>
        /// <returns></returns>
        public MAV MAV(int mAPeriod, int typeInt)
        {
            return MAV(Input, mAPeriod, typeInt);
        }

        /// <summary>
        /// Variable moving average
        /// </summary>
        /// <returns></returns>
        public MAV MAV(Data.IDataSeries input, int mAPeriod, int typeInt)
        {
            checkMAV.MAPeriod = mAPeriod;
            mAPeriod = checkMAV.MAPeriod;
            checkMAV.TypeInt = typeInt;
            typeInt = checkMAV.TypeInt;

            if (cacheMAV != null)
                for (int idx = 0; idx < cacheMAV.Length; idx++)
                    if (cacheMAV[idx].MAPeriod == mAPeriod && cacheMAV[idx].TypeInt == typeInt && cacheMAV[idx].EqualsInput(input))
                        return cacheMAV[idx];

            MAV indicator = new MAV();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.MAPeriod = mAPeriod;
            indicator.TypeInt = typeInt;

            MAV[] tmp = new MAV[cacheMAV == null ? 1 : cacheMAV.Length + 1];
            if (cacheMAV != null)
                cacheMAV.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheMAV = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Variable moving average
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MAV MAV(int mAPeriod, int typeInt)
        {
            return _indicator.MAV(Input, mAPeriod, typeInt);
        }

        /// <summary>
        /// Variable moving average
        /// </summary>
        /// <returns></returns>
        public Indicator.MAV MAV(Data.IDataSeries input, int mAPeriod, int typeInt)
        {
            return _indicator.MAV(input, mAPeriod, typeInt);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Variable moving average
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.MAV MAV(int mAPeriod, int typeInt)
        {
            return _indicator.MAV(Input, mAPeriod, typeInt);
        }

        /// <summary>
        /// Variable moving average
        /// </summary>
        /// <returns></returns>
        public Indicator.MAV MAV(Data.IDataSeries input, int mAPeriod, int typeInt)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.MAV(input, mAPeriod, typeInt);
        }

    }
}
#endregion
