// 
// Copyright (C) 2006, NinjaTrader LLC <ninjatrader@ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Pivot Points enhanced by cvax.
	/// </summary>
	[Description("Pivot Points to R5/S5 by cvax")]
	[Gui.Design.DisplayName("Pivot Points Enhanced")]
	public class PivotPoints : Indicator
	{
		#region Variables
		private	SolidBrush[]	brushes			= { new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
													new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
													new SolidBrush(Color.Black)};
		private	double			currentClose	= 0;
		private DateTime		currentDate		= Cbi.Globals.MinDate;
		private DateTime		currentMonth	= Cbi.Globals.MinDate;
		private DateTime		currentWeek		= Cbi.Globals.MinDate;
		private	double			currentHigh		= double.MinValue;
		private	double			currentLow		= double.MaxValue;
		private Data.PivotRange pivotRangeType	= PivotRange.Daily;
		private bool			labels			= true;
		private	double			pp				= 0;
		private	double			r1				= 0;
		private	double			r2				= 0;
		private	double			r3				= 0;
		private	double			r4				= 0;
		private	double			r5				= 0;
		private	double			s1				= 0;
		private	double			s2				= 0;
		private	double			s3				= 0;
		private	double			s4				= 0;
		private	double			s5				= 0;
		private	StringFormat	stringFormat	= new StringFormat();
		private int				width			= 370;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
				Add(new Plot(Color.Red,		"R5"));
				Add(new Plot(Color.Red,		"R4"));
				Add(new Plot(Color.Red,		"R3"));
				Add(new Plot(Color.Red,		"R2"));
				Add(new Plot(Color.Red,		"R1"));
				Add(new Plot(Color.DarkGreen,	"PP"));
				Add(new Plot(Color.Blue,	"S1"));
				Add(new Plot(Color.Blue,	"S2"));
				Add(new Plot(Color.Blue,	"S3"));
				Add(new Plot(Color.Blue,	"S4"));
				Add(new Plot(Color.Blue,	"S5"));
			
			Plots[0].Pen = new Pen(Color.Red, 2);
			Plots[1].Pen = new Pen(Color.Red, 2);
			Plots[2].Pen = new Pen(Color.Red, 2);
			Plots[3].Pen = new Pen(Color.Red, 2);
			Plots[4].Pen = new Pen(Color.Red, 2);
			Plots[5].Pen = new Pen(Color.Black, 2);
			Plots[6].Pen = new Pen(Color.Blue, 2);
			Plots[7].Pen = new Pen(Color.Blue, 2);
			Plots[8].Pen = new Pen(Color.Blue, 2);
			Plots[9].Pen = new Pen(Color.Blue, 2);
			Plots[10].Pen = new Pen(Color.Blue, 2);
			
			AutoScale				= false;
			CalculateOnBarClose		= false;
			Overlay					= true;
			stringFormat.Alignment	= StringAlignment.Far;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (Bars == null)
				return; 
			if (!Session.IsIntraday(Bars.Period.Id) && Bars.Period.Id != PeriodType.Day)
				return;
			if (Bars.Period.Id == PeriodType.Day && pivotRangeType == PivotRange.Daily)
				return;
			if (Bars.Period.Id == PeriodType.Day && Bars.Period.Value > 1)
				return;

			if ((currentDate != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Daily && Time[0].Date != currentDate)
				|| (currentWeek != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Weekly && RoundUpTimeToPeriodTime(Time[0].Date, PivotRange.Weekly) != currentWeek) 
				|| (currentMonth != Cbi.Globals.MinDate && pivotRangeType == PivotRange.Monthly && RoundUpTimeToPeriodTime(Time[0].Date, PivotRange.Monthly) != currentMonth)) 
			{
				pp				= (currentHigh + currentLow + currentClose) / 3;
				s1				= 2 * pp - currentHigh;
				r1				= 2 * pp - currentLow;
				s2				= pp - (currentHigh - currentLow);
				r2				= pp + (currentHigh - currentLow);
				s3				= pp - 2*(currentHigh - currentLow);
				r3				= pp + 2*(currentHigh - currentLow);
				s4				= pp - 3*(currentHigh - currentLow);
				r4				= pp + 3*(currentHigh - currentLow);
				s5				= pp - 4*(currentHigh - currentLow);
				r5				= pp + 4*(currentHigh - currentLow);
				currentClose	= Close[0]; 
				currentHigh		= High[0];
				currentLow		= Low[0];
			}
			else
			{
				currentClose	= Close[0]; 
				currentHigh		= Math.Max(currentHigh, High[0]);
				currentLow		= Math.Min(currentLow, Low[0]);
			}
			if (pivotRangeType == PivotRange.Daily)
				currentDate = Time[0].Date;
			if (pivotRangeType == PivotRange.Weekly)
				currentWeek = RoundUpTimeToPeriodTime(Time[0].Date, PivotRange.Weekly);
			if (pivotRangeType == PivotRange.Monthly)
				currentMonth = RoundUpTimeToPeriodTime(Time[0].Date, PivotRange.Monthly);

			if (pp != 0)
			{
				PP.Set(pp);
				R1.Set(r1);
				S1.Set(s1);
				R2.Set(r2);
				S2.Set(s2);
				R3.Set(r3);
				S3.Set(s3);
				R4.Set(r4);
				S4.Set(s4);
				R5.Set(r5);
				S5.Set(s5);
			}
		}

		#region Properties
		/// <summary>
		/// </summary>
		[Description("Type of period for pivot points.")]
		[Category("Parameters")]
		public Data.PivotRange PivotRangeType 
		{
			get { return pivotRangeType; }
			set { pivotRangeType = value; }
		}

		/// <summary>
		/// </summary>
		[Description("Turn Pivot Labels On/Off.")]
		[Category("Pivot Line Settings")]
		public bool Labels 
		{
			get { return labels; }
			set { labels = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("# of bars back the Pivot Line will be drawn.")]
		[Category("Pivot Line Settings")]
		public int Width
		{
			get { return width; }
			set { width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PP
		{
			get { return Values[5]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R1
		{
			get { return Values[4]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R2
		{
			get { return Values[3]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R3
		{
			get { return Values[2]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R4
		{
			get { return Values[1]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R5
		{
			get { return Values[0]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S1
		{
			get { return Values[6]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S2
		{
			get { return Values[7]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S3
		{
			get { return Values[8]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S4
		{
			get { return Values[9]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S5
		{
			get { return Values[10]; }
		}
		#endregion

		#region Miscellaneous
		private int GetDayOfWeek(DateTime date)
		{
			DateTime saturday = new DateTime(1776, 7, 4).AddDays(2);
			TimeSpan diff = date.Subtract(saturday);
			return (diff.Days % 7);
		}

		private DateTime RoundUpTimeToPeriodTime(DateTime time, PivotRange pivotRange)
		{
			if (pivotRange == PivotRange.Weekly)
			{
				DateTime periodStart = time.AddDays((6-GetDayOfWeek(time)));
				return periodStart.Date.AddDays(System.Math.Ceiling(System.Math.Ceiling(time.Date.Subtract(periodStart.Date).TotalDays) / 7) * 7).Date;
			}
			else if (pivotRange == PivotRange.Monthly)
			{
				DateTime result = new DateTime(time.Year, time.Month, 1); 
				return result.AddMonths(1).AddDays(-1);
			}
			else
				return time;
		}

		/// <summary>
		/// </summary>
		/// <param name="graphics"></param>
		/// <param name="bounds"></param>
		/// <param name="min"></param>
		/// <param name="max"></param>
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars == null || ChartControl == null)
				return;

			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			for (int seriesCount = 0; seriesCount < Values.Length; seriesCount++)
			{
				SolidBrush		brush				= brushes[seriesCount];
				DateTime		lastDate			= Cbi.Globals.MinDate;
				DateTime		lastWeek			= Cbi.Globals.MinDate;
				DateTime		lastMonth			= Cbi.Globals.MinDate;
				int				firstX				= -1;
				int				lastX				= -1;
				int				lastY				= -1;
				int				xcounter			= 0;
				SmoothingMode	oldSmoothingMode	= graphics.SmoothingMode;
				GraphicsPath	path				= new GraphicsPath();
				Gui.Chart.Plot	plot				= Plots[seriesCount];
				DataSeries	series				= (DataSeries) Values[seriesCount];
				
				if (brush.Color != plot.Pen.Color)	
					brush = new SolidBrush(plot.Pen.Color);

				for (int count = ChartControl.BarsPainted - 1; count >= ChartControl.BarsPainted - Math.Min(ChartControl.BarsPainted, Width); count--)
				{
					
					int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + count;
					if (idx - Displacement < 0 || idx - Displacement >= Bars.Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
						continue;
					else if (!series.IsValidPlot(idx))
						continue;

					if (pivotRangeType == PivotRange.Daily && lastDate == Cbi.Globals.MinDate) 
						lastDate = Bars.Get(idx).Time.Date;
					else if (pivotRangeType == PivotRange.Weekly && lastWeek == Cbi.Globals.MinDate) 
						lastWeek = RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Weekly);
					else if (pivotRangeType == PivotRange.Monthly && lastMonth == Cbi.Globals.MinDate) 
						lastMonth = RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Monthly);
					else if (pivotRangeType == PivotRange.Daily && lastDate != Bars.Get(idx).Time.Date 
							|| pivotRangeType == PivotRange.Weekly && lastWeek != RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Weekly)
							|| pivotRangeType == PivotRange.Monthly && lastMonth != RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Monthly))
						continue;

					double		val = series.Get(idx);
					int			x	= (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2
										- (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + count * ChartControl.BarSpace);
					int			y	= (int) ((bounds.Y + bounds.Height) - ((val - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);

					if (xcounter == 0)
					{
						firstX	= x;
					}
					if (lastX >= 0)
					{
						if (y != lastY) // Problem here is, that last bar of old day has date of new day
							y = lastY;
						path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
					}
					lastX	= x;
					lastY	= y;
					
					xcounter++;
				}

				graphics.SmoothingMode = SmoothingMode.AntiAlias;
				graphics.DrawPath(plot.Pen, path);
				graphics.SmoothingMode = oldSmoothingMode;
				if ( labels == true )
				{
					if (pivotRangeType == PivotRange.Daily)
					{
						graphics.DrawString(plot.Name, ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
						//graphics.DrawString(plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left
					}
					else if (pivotRangeType == PivotRange.Weekly)
					{
						graphics.DrawString("Weekly " + plot.Name, ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
						//graphics.DrawString("Weekly " + plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left
					}
					else
					{
						graphics.DrawString("Monthly " + plot.Name, ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
						//graphics.DrawString("Monthly " + plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left
					}
				}
				else
				{
					graphics.DrawString("", ChartControl.Font, brush, firstX, lastY - ChartControl.Font.GetHeight(), stringFormat);			//align label right
					//graphics.DrawString("", ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormat);		//align label left

				}
			}
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private PivotPoints[] cachePivotPoints = null;

        private static PivotPoints checkPivotPoints = new PivotPoints();

        /// <summary>
        /// Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        public PivotPoints PivotPoints()
        {
            return PivotPoints(Input);
        }

        /// <summary>
        /// Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        public PivotPoints PivotPoints(Data.IDataSeries input)
        {

            if (cachePivotPoints != null)
                for (int idx = 0; idx < cachePivotPoints.Length; idx++)
                    if (cachePivotPoints[idx].EqualsInput(input))
                        return cachePivotPoints[idx];

            PivotPoints indicator = new PivotPoints();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;

            PivotPoints[] tmp = new PivotPoints[cachePivotPoints == null ? 1 : cachePivotPoints.Length + 1];
            if (cachePivotPoints != null)
                cachePivotPoints.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cachePivotPoints = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PivotPoints PivotPoints()
        {
            return _indicator.PivotPoints(Input);
        }

        /// <summary>
        /// Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        public Indicator.PivotPoints PivotPoints(Data.IDataSeries input)
        {
            return _indicator.PivotPoints(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.PivotPoints PivotPoints()
        {
            return _indicator.PivotPoints(Input);
        }

        /// <summary>
        /// Pivot Points to R5/S5 by cvax
        /// </summary>
        /// <returns></returns>
        public Indicator.PivotPoints PivotPoints(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.PivotPoints(input);
        }

    }
}
#endregion
