
#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
	/* v1 Dec 2008
	Removed Decimal variable - no longer required
	Altered string format for high and low
	Added Trim property to price display
	*/
	
{
    public class highlow : Indicator
    {
        #region Variables
           // private int Decimal = 0; // Default setting for dec places
			private int Offset = 2;
			private int x = 0;
			private bool ShowClose = true;
			private int Chars 			= 2;// Characters to remove
			private int NBar = 0;// Defualt is CurrentBar
			private	System.Drawing.Font		txtFont;
			private int	fontSize        = 10;
        #endregion

        protected override void Initialize()
        {
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			txtFont		= new Font("Verdana Ref", fontSize);
			if (offset < 0 )  // Text behind CurrentBar
			{x = Math.Abs(offset);}
			else
			{x = 0; }         // Text At Current bar
			
        }

        protected override void OnBarUpdate()
        {
			if (Historical)  return;
			
			string finallow = String.Format("{0}",( Low[NBar])).Substring(chars);
			string finalhigh = String.Format("{0}",( High[NBar])).Substring(Chars);
			DrawText("HighText", false,  finalhigh , x, High[0]+ (TickSize * Math.Abs( Offset)) ,Color.Blue, txtFont, StringAlignment.Near, Color.Transparent, Color.Transparent, 50);
			DrawText("LowText", false,  finallow, x, Low[0] - (TickSize*Math.Abs( Offset)),Color.Red,  txtFont, StringAlignment.Near, Color.Transparent, Color.Transparent, 50);
			
			if(ShowClose)// "<"
			{
			DrawText("Closemarker", false,  "  " +"<", 0, Close[0] ,Color.Red,  txtFont, StringAlignment.Near, Color.Transparent, Color.Transparent, 50);
			}
        }

        #region Properties



		[Description("TICKSIZE offset from Current High/Low")]
        [Category("Parameters")]
		[NinjaTrader.Gui.Design.DisplayName("Tick Offset")]
        public int offset
        {
            get { return Offset; }
            set { Offset = Math.Max(-5000, value); }
        }
		
		[Description("CurrentBar = Default, 1 = prior Bar , etc")]
        [Category("Parameters")]
		[NinjaTrader.Gui.Design.DisplayName("Show Bar No")]

        public int nbar
        {
            get { return NBar; }
            set { NBar = Math.Max(0, value); }
        }
		
        [Description("<  follows last price")]
        [Category("Parameters")]
		[NinjaTrader.Gui.Design.DisplayName("Show Close Price")]
        public bool showclose
        {
            get { return ShowClose; }
            set { ShowClose = value; }
        }
		[Description("Font Size for text only")]
		[Category("Status panel")]
		[NinjaTrader.Gui.Design.DisplayName("Font size")]
		public int StatusPanelFontSize
		{
			get { return fontSize; }
			set { fontSize = Math.Max(1,value); }
		}
		[Description("Characters removed from the lhs of price display")]
        [Category("Parameters")]
		[NinjaTrader.Gui.Design.DisplayName("Chars to Remove")]

        public int chars
        {
            get { return Chars; }
            set { Chars = Math.Max(0, value); }
       }	
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private highlow[] cachehighlow = null;

        private static highlow checkhighlow = new highlow();

      
        public highlow highlow( int offset)
        {
            return highlow(Input, offset);
        }

    
        public highlow highlow(Data.IDataSeries input, int offset)
        {
          
            checkhighlow.offset = offset;
            offset = checkhighlow.offset;

            if (cachehighlow != null)
                for (int idx = 0; idx < cachehighlow.Length; idx++)
                    if ( cachehighlow[idx].offset == offset && cachehighlow[idx].EqualsInput(input))
                        return cachehighlow[idx];

            highlow indicator = new highlow();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
       
            indicator.offset = offset;
            indicator.SetUp();

            highlow[] tmp = new highlow[cachehighlow == null ? 1 : cachehighlow.Length + 1];
            if (cachehighlow != null)
                cachehighlow.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cachehighlow = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.highlow highlow( int offset)
        {
            return _indicator.highlow(Input, offset);
        }

    
        public Indicator.highlow highlow(Data.IDataSeries input,  int offset)
        {
            return _indicator.highlow(input, offset);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.highlow highlow(int offset)
        {
            return _indicator.highlow(Input, offset);
        }

    
        public Indicator.highlow highlow(Data.IDataSeries input,  int offset)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.highlow(input, offset);
        }

    }
}
#endregion
